SHELL=bash
CALL_DATE:=$(shell date +%m%d%y-%H%M%S)
DATAPATH = ../data/$(DATA)
SUBMITDIR = /opt/xmt/class/submit/cmsc751/summation
CORRECTRESULT = ../data/$(DATA)/correctout.txt
DUMP_VAR = sum
DUMP_FILE_PREFIX = dump_$(INPUT)_$(DATA)
DUMP_FILE=$(DUMP_FILE_PREFIX).$(DUMP_VAR)

PROGRAM ?= $(INPUT)
TEMP_FILE=tmp_$(INPUT)_$(DATA).txt

override COMPILER_PARAMS += -sim

help:
	@echo "USAGE: "
	@echo "  make help    : Displays this information."
	@echo "  make clean   : Removes all files created during a compilation or a run."
	@echo "  make compile : Compiles the input file with the given data set."
	@echo "  make run     : Compiles and runs the input file with the given data set."
	@echo "  make check   : Compiles, runs and compares the output against the correct"
	@echo "                 output."
	@echo "  make testall : Checks the correctness of merge.s.c, merge.p.c and mergesort.c"
	@echo "                 against all the data sets. Creates a testlog.txt file. Search"
	@echo "                 this file for errors in order to see the cases that fail. Note"
	@echo "                 that this command will not quit on error, it will continue until"
	@echo "                 it runs all the cases."
	@echo "                 Overuse of this command will overload the FPGA and will block"
	@echo "                 other users so use judiciously."
	@echo "  make submit  : Submits the assignment package for grading. The input will be"
	@echo "                 compressed into st09<id>_<date>.tar.gz which will be placed under"
	@echo "                 $(SUBMITDIR)."
	@echo "                 If you use this command multiple times multiple files under the."
	@echo "                 submission directory will be created. Only the file with the most"
	@echo "                 recent date will be graded."
	@echo "                 Run 'make submitcheck' before this command."
	@echo "  make submitcheck : Checks if all the deliverables for submission are present."
	@echo ""
	@echo "  Compile, run and check commands require setting the INPUT (or PROGRAM) and DATA"
	@echo "  arguments on command line."
	@echo "  Following are some examples:"
	@echo ""
	@echo "    make check INPUT=ser-s.c DATA=dm1"
	@echo "    make run INPUT=bbt-s.c DATA=dm3"
	@echo "    make testall"
#	@echo "    make crossover DATA=dms2"

run: compile runsim 

check: compile runsim_dump checkresult

run_stats: compile	
	@echo "xmtsim -cycle -count $(patsubst %.c,%,$(PROGRAM)).sim -binload $(patsubst %.c,%,$(PROGRAM)).b"
	@xmtsim -cycle -count $(patsubst %.c,%,$(PROGRAM)).sim -binload $(patsubst %.c,%,$(PROGRAM)).b ; \
	 [[ $$? != 0 ]] && echo "ERROR: Run on the simulator failed!" && exit 1 || exit 0


runsim_dump: 
	@echo "xmtsim -cycle $(patsubst %.c,%,$(PROGRAM)).sim -binload $(patsubst %.c,%,$(PROGRAM)).b -textdump $(DUMP_FILE_PREFIX) -dumpvar $(DUMP_VAR)"
	@xmtsim -cycle $(patsubst %.c,%,$(PROGRAM)).sim -binload $(patsubst %.c,%,$(PROGRAM)).b -textdump $(DUMP_FILE_PREFIX) -dumpvar $(DUMP_VAR) ; \
	 [[ $$? != 0 ]] && echo "ERROR: Run on the simulator failed!" && exit 1 || exit 0

runsim: 
	@echo "xmtsim -cycle $(patsubst %.c,%,$(PROGRAM)).sim -binload $(patsubst %.c,%,$(PROGRAM)).b"
	@xmtsim -cycle $(patsubst %.c,%,$(PROGRAM)).sim -binload $(patsubst %.c,%,$(PROGRAM)).b ; \
	 [[ $$? != 0 ]] && echo "ERROR: Run on the simulator failed!" && exit 1 || exit 0

compile: checkparams
	@echo "xmtcc $(COMPILER_PARAMS) -include $(DATAPATH)/*.h $(DATAPATH)/*.xbo $(PROGRAM) -o $(patsubst %.c,%,$(PROGRAM))"
	@xmtcc $(COMPILER_PARAMS) -include $(DATAPATH)/*.h $(DATAPATH)/*.xbo $(PROGRAM) -o $(patsubst %.c,%,$(PROGRAM)); \
	 [[ $$? != 0 ]] && echo "ERROR: Compilation failed!" && exit 1 || exit 0

checkresult: 
# Get rid of newlines and whitespace first
	@cat $(DUMP_FILE) |sed 's/[[:space:]][[:space:]]*/\n/g'|egrep -v "^$$"> $(TEMP_FILE)
	@mv $(TEMP_FILE) $(DUMP_FILE) 
	@echo "diff -bwi $(DUMP_FILE) $(CORRECTRESULT) >/dev/null"
	@diff -bwi $(DUMP_FILE) $(CORRECTRESULT) >/dev/null; \
	 [[ $$? != 0 ]] && echo "ERROR: $(DUMP_FILE) does not match $(CORRECTRESULT)!" \
		&& exit 1 || exit 0
	@echo "PASS: Your program passed the correctness test!"

checkparams:
	@if [[ "$(PROGRAM)" == "" ]]; then \
		echo "ERROR: PROGRAM (or INPUT) variable is not set."; \
		exit 1; \
	fi
	@if ! [[ -e $(PROGRAM) ]]; then \
		echo "ERROR: $(PROGRAM) does not exist."; \
		exit 1; \
	fi
	@if [[ "$(DATA)" == "" ]]; then \
		echo "ERROR: DATA variable is not set."; \
		exit 1; \
	fi
	@if ! [[ -e $(DATAPATH) ]]; then \
		echo "ERROR: Data $(DATA) does not exist."; \
		exit 1; \
	fi

testall:
	$(MAKE) subtestall 2>&1| tee testlog.txt

subtestall:
	-$(MAKE) check COMPILER_PARAMS="-quiet" PROGRAM=bbt-s.c DATA=dm1 
	-$(MAKE) check COMPILER_PARAMS="-quiet" PROGRAM=bbt-s.c DATA=dm2
	-$(MAKE) check COMPILER_PARAMS="-quiet" PROGRAM=bbt-s.c DATA=dm3
	-$(MAKE) check COMPILER_PARAMS="-quiet" PROGRAM=best-s.c DATA=dm1 
	-$(MAKE) check COMPILER_PARAMS="-quiet" PROGRAM=best-s.c DATA=dm2
	-$(MAKE) check COMPILER_PARAMS="-quiet" PROGRAM=best-s.c DATA=dm3
	-$(MAKE) check COMPILER_PARAMS="-quiet" PROGRAM=bkbt-s.c DATA=dm1 
	-$(MAKE) check COMPILER_PARAMS="-quiet" PROGRAM=bkbt-s.c DATA=dm2
	-$(MAKE) check COMPILER_PARAMS="-quiet" PROGRAM=bkbt-s.c DATA=dm3
	-$(MAKE) check COMPILER_PARAMS="-quiet" PROGRAM=nbw-s.c DATA=dm1 
	-$(MAKE) check COMPILER_PARAMS="-quiet" PROGRAM=nbw-s.c DATA=dm2
	-$(MAKE) check COMPILER_PARAMS="-quiet" PROGRAM=nbw-s.c DATA=dm3
	-$(MAKE) check COMPILER_PARAMS="-quiet" PROGRAM=pser-s.c DATA=dm1 
	-$(MAKE) check COMPILER_PARAMS="-quiet" PROGRAM=pser-s.c DATA=dm2
	-$(MAKE) check COMPILER_PARAMS="-quiet" PROGRAM=pser-s.c DATA=dm3
	-$(MAKE) check COMPILER_PARAMS="-quiet" PROGRAM=ser-s.c DATA=dm1 
	-$(MAKE) check COMPILER_PARAMS="-quiet" PROGRAM=ser-s.c DATA=dm2
	-$(MAKE) check COMPILER_PARAMS="-quiet" PROGRAM=ser-s.c DATA=dm3

submitcheck:
	@if ! [[ -e bbt-s.c ]]; then echo "FAIL: bbt-s.c not found."; else \
		echo "OK: Found bbt-s.c"; fi
	@if ! [[ -e best-s.c ]]; then echo "FAIL: best-s.c not found."; else \
		echo "OK: Found best-s.c"; fi
	@if ! [[ -e best-s.txt ]]; then echo "FAIL: best-s.txt not found."; else \
		echo "OK: Found best-s.txt"; fi
	@if ! [[ -e bkbt-s.c ]]; then echo "FAIL: bkbt-s.c not found."; else \
		echo "OK: Found bkbt-s.c"; fi
	@if ! [[ -e nbw-s.c ]]; then echo "FAIL: nbw-s.c not found."; else \
		echo "OK: Found nbw-s.c"; fi
	@if ! [[ -e pser-s.c ]]; then echo "FAIL: pser-s.c not found."; else \
		echo "OK: Found pser-s.c"; fi
	@if ! [[ -e ser-s.c ]]; then echo "FAIL: ser-s.c not found."; else \
		echo "OK: Found ser-s.c"; fi
	@if [[ -e bkbt-s.c ]]; then \
	    grep '#define KARY' bkbt-s.c|sed 's/.*KARY//g'| \
			awk '{if($$1 ~ /^[0-9][0-9]*$$/) print "OK: Reported KARY value: " $$1; else print "FAIL: No KARY value reported.";}'; \
	 else \
		echo "FAIL: No KARY value reported."; \
	 fi; \


submit:	
# FIXME: Use ssh to submit from the VBOX
	@cd ..; tarfile=$(LOGNAME)_$(CALL_DATE).tgz; \
	 tar czvf $${tarfile} `find . -type f -print`; \
	./scripts/submit_ssh.pl $${tarfile} ${SUBMITDIR}  ; \
	rm $${tarfile}

clean:
	rm -rf *.b *.sim $(DUMP_FILE) *~ *.log output*.txt

